@@ -12,6 +12,7 @@ These tools are used by `matplotlib.backend_managers.ToolManager`
 """
 
 import enum
+import functools
 import re
 import time
 from types import SimpleNamespace
@@ -36,6 +37,35 @@ class Cursors(enum.IntEnum):  # Must subclass int for the macOS backend.
     RESIZE_VERTICAL = enum.auto()
 cursors = Cursors  # Backcompat.
 
+
+# _tool_registry, _register_tool_class, and _find_tool_class implement a
+# mechanism through which ToolManager.add_tool can determine whether a subclass
+# of the requested tool class has been registered (either for the current
+# canvas class or for a parent class), in which case that tool subclass will be
+# instantiated instead.  This is the mechanism used e.g. to allow different
+# GUI backends to implement different specializations for ConfigureSubplots.
+
+
+_tool_registry = set()
+
+
+def _register_tool_class(canvas_cls, tool_cls=None):
+    """Decorator registering *tool_cls* as a tool class for *canvas_cls*."""
+    if tool_cls is None:
+        return functools.partial(_register_tool_class, canvas_cls)
+    _tool_registry.add((canvas_cls, tool_cls))
+    return tool_cls
+
+
+def _find_tool_class(canvas_cls, tool_cls):
+    """Find a subclass of *tool_cls* registered for *canvas_cls*."""
+    for canvas_parent in canvas_cls.__mro__:
+        for tool_child in _api.recursive_subclasses(tool_cls):
+            if (canvas_parent, tool_child) in _tool_registry:
+                return tool_child
+    return tool_cls
+
+
 # Views positions tool
 _views_positions = 'viewpos'
 
@@ -53,7 +83,9 @@ class ToolBase:
     Keymap to associate with this tool.
 
     ``list[str]``: List of keys that will trigger this tool when a keypress
-    event is emitted on ``self.figure.canvas``.
+    event is emitted on ``self.figure.canvas``.  Note that this attribute is
+    looked up on the instance, and can therefore be a property (this is used
+    e.g. by the built-in tools to load the rcParams at instantiation time).
     """
 
     description = None
@@ -123,6 +155,7 @@ class ToolBase:
         """
         pass
 
+    @_api.deprecated("3.6", alternative="tool_removed_event")
     def destroy(self):
         """
         Destroy the tool.
@@ -322,7 +355,7 @@ class ToolCursorPosition(ToolBase):
 
 class RubberbandBase(ToolBase):
     """Draw and remove a rubberband."""
-    def trigger(self, sender, event, data):
+    def trigger(self, sender, event, data=None):
         """Call `draw_rubberband` or `remove_rubberband` based on data."""
         if not self.figure.canvas.widgetlock.available(sender):
             return
@@ -352,7 +385,7 @@ class ToolQuit(ToolBase):
     """Tool to call the figure manager destroy method."""
 
     description = 'Quit the figure'
-    default_keymap = mpl.rcParams['keymap.quit']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.quit'])
 
     def trigger(self, sender, event, data=None):
         Gcf.destroy_fig(self.figure)
@@ -362,7 +395,7 @@ class ToolQuitAll(ToolBase):
     """Tool to call the figure manager destroy method."""
 
     description = 'Quit all figures'
-    default_keymap = mpl.rcParams['keymap.quit_all']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.quit_all'])
 
     def trigger(self, sender, event, data=None):
         Gcf.destroy_all()
@@ -372,7 +405,7 @@ class ToolGrid(ToolBase):
     """Tool to toggle the major grids of the figure."""
 
     description = 'Toggle major grids'
-    default_keymap = mpl.rcParams['keymap.grid']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.grid'])
 
     def trigger(self, sender, event, data=None):
         sentinel = str(uuid.uuid4())
@@ -387,7 +420,7 @@ class ToolMinorGrid(ToolBase):
     """Tool to toggle the major and minor grids of the figure."""
 
     description = 'Toggle major and minor grids'
-    default_keymap = mpl.rcParams['keymap.grid_minor']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.grid_minor'])
 
     def trigger(self, sender, event, data=None):
         sentinel = str(uuid.uuid4())
@@ -398,16 +431,13 @@ class ToolMinorGrid(ToolBase):
             mpl.backend_bases.key_press_handler(event, self.figure.canvas)
 
 
-class ToolFullScreen(ToolToggleBase):
+class ToolFullScreen(ToolBase):
     """Tool to toggle full screen."""
 
     description = 'Toggle fullscreen mode'
-    default_keymap = mpl.rcParams['keymap.fullscreen']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.fullscreen'])
 
-    def enable(self, event):
-        self.figure.canvas.manager.full_screen_toggle()
-
-    def disable(self, event):
+    def trigger(self, sender, event, data=None):
         self.figure.canvas.manager.full_screen_toggle()
 
 
@@ -419,11 +449,11 @@ class AxisScaleBase(ToolToggleBase):
             return
         super().trigger(sender, event, data)
 
-    def enable(self, event):
+    def enable(self, event=None):
         self.set_scale(event.inaxes, 'log')
         self.figure.canvas.draw_idle()
 
-    def disable(self, event):
+    def disable(self, event=None):
         self.set_scale(event.inaxes, 'linear')
         self.figure.canvas.draw_idle()
 
@@ -432,7 +462,7 @@ class ToolYScale(AxisScaleBase):
     """Tool to toggle between linear and logarithmic scales on the Y axis."""
 
     description = 'Toggle scale Y axis'
-    default_keymap = mpl.rcParams['keymap.yscale']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.yscale'])
 
     def set_scale(self, ax, scale):
         ax.set_yscale(scale)
@@ -442,7 +472,7 @@ class ToolXScale(AxisScaleBase):
     """Tool to toggle between linear and logarithmic scales on the X axis."""
 
     description = 'Toggle scale X axis'
-    default_keymap = mpl.rcParams['keymap.xscale']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.xscale'])
 
     def set_scale(self, ax, scale):
         ax.set_xscale(scale)
@@ -596,7 +626,7 @@ class ToolHome(ViewsPositionsBase):
 
     description = 'Reset original view'
     image = 'home'
-    default_keymap = mpl.rcParams['keymap.home']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.home'])
     _on_trigger = 'home'
 
 
@@ -605,7 +635,7 @@ class ToolBack(ViewsPositionsBase):
 
     description = 'Back to previous view'
     image = 'back'
-    default_keymap = mpl.rcParams['keymap.back']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.back'])
     _on_trigger = 'back'
 
 
@@ -614,7 +644,7 @@ class ToolForward(ViewsPositionsBase):
 
     description = 'Forward to next view'
     image = 'forward'
-    default_keymap = mpl.rcParams['keymap.forward']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.forward'])
     _on_trigger = 'forward'
 
 
@@ -630,7 +660,7 @@ class SaveFigureBase(ToolBase):
 
     description = 'Save the figure'
     image = 'filesave'
-    default_keymap = mpl.rcParams['keymap.save']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.save'])
 
 
 class ZoomPanBase(ToolToggleBase):
@@ -646,7 +676,7 @@ class ZoomPanBase(ToolToggleBase):
         self.scrollthresh = .5  # .5 second scroll threshold
         self.lastscroll = time.time()-self.scrollthresh
 
-    def enable(self, event):
+    def enable(self, event=None):
         """Connect press/release events and lock the canvas."""
         self.figure.canvas.widgetlock(self)
         self._idPress = self.figure.canvas.mpl_connect(
@@ -656,7 +686,7 @@ class ZoomPanBase(ToolToggleBase):
         self._idScroll = self.figure.canvas.mpl_connect(
             'scroll_event', self.scroll_zoom)
 
-    def disable(self, event):
+    def disable(self, event=None):
         """Release the canvas and disconnect press/release events."""
         self._cancel_action()
         self.figure.canvas.widgetlock.release(self)
@@ -705,7 +735,7 @@ class ToolZoom(ZoomPanBase):
 
     description = 'Zoom to rectangle'
     image = 'zoom_to_rect'
-    default_keymap = mpl.rcParams['keymap.zoom']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.zoom'])
     cursor = cursors.SELECT_REGION
     radio_group = 'default'
 
@@ -791,7 +821,7 @@ class ToolZoom(ZoomPanBase):
             self._cancel_action()
             return
 
-        last_a = []
+        done_ax = []
 
         for cur_xypress in self._xypress:
             x, y = event.x, event.y
@@ -802,14 +832,9 @@ class ToolZoom(ZoomPanBase):
                 return
 
             # detect twinx, twiny axes and avoid double zooming
-            twinx, twiny = False, False
-            if last_a:
-                for la in last_a:
-                    if a.get_shared_x_axes().joined(a, la):
-                        twinx = True
-                    if a.get_shared_y_axes().joined(a, la):
-                        twiny = True
-            last_a.append(a)
+            twinx = any(a.get_shared_x_axes().joined(a, a1) for a1 in done_ax)
+            twiny = any(a.get_shared_y_axes().joined(a, a1) for a1 in done_ax)
+            done_ax.append(a)
 
             if self._button_pressed == 1:
                 direction = 'in'
@@ -829,7 +854,7 @@ class ToolZoom(ZoomPanBase):
 class ToolPan(ZoomPanBase):
     """Pan axes with left mouse, zoom with right."""
 
-    default_keymap = mpl.rcParams['keymap.pan']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.pan'])
     description = 'Pan axes with left mouse, zoom with right'
     image = 'move'
     cursor = cursors.MOVE
@@ -894,7 +919,7 @@ class ToolPan(ZoomPanBase):
 
 class ToolHelpBase(ToolBase):
     description = 'Print tool list, shortcuts and description'
-    default_keymap = mpl.rcParams['keymap.help']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.help'])
     image = 'help'
 
     @staticmethod
@@ -934,7 +959,7 @@ class ToolCopyToClipboardBase(ToolBase):
     """Tool to copy the figure to the clipboard."""
 
     description = 'Copy the canvas figure to clipboard'
-    default_keymap = mpl.rcParams['keymap.copy']
+    default_keymap = property(lambda self: mpl.rcParams['keymap.copy'])
 
     def trigger(self, *args, **kwargs):
         message = "Copy tool is not available"
@@ -943,8 +968,8 @@ class ToolCopyToClipboardBase(ToolBase):
 
 default_tools = {'home': ToolHome, 'back': ToolBack, 'forward': ToolForward,
                  'zoom': ToolZoom, 'pan': ToolPan,
-                 'subplots': 'ToolConfigureSubplots',
-                 'save': 'ToolSaveFigure',
+                 'subplots': ConfigureSubplotsBase,
+                 'save': SaveFigureBase,
                  'grid': ToolGrid,
                  'grid_minor': ToolMinorGrid,
                  'fullscreen': ToolFullScreen,
@@ -954,17 +979,15 @@ default_tools = {'home': ToolHome, 'back': ToolBack, 'forward': ToolForward,
                  'yscale': ToolYScale,
                  'position': ToolCursorPosition,
                  _views_positions: ToolViewsPositions,
-                 'cursor': 'ToolSetCursor',
-                 'rubberband': 'ToolRubberband',
-                 'help': 'ToolHelp',
-                 'copy': 'ToolCopyToClipboard',
+                 'cursor': SetCursorBase,
+                 'rubberband': RubberbandBase,
+                 'help': ToolHelpBase,
+                 'copy': ToolCopyToClipboardBase,
                  }
-"""Default tools"""
 
 default_toolbar_tools = [['navigation', ['home', 'back', 'forward']],
                          ['zoompan', ['pan', 'zoom', 'subplots']],
                          ['io', ['save', 'help']]]
-"""Default tools in the toolbar"""
 
 
 def add_tools_to_manager(toolmanager, tools=default_tools):
@@ -976,8 +999,8 @@ def add_tools_to_manager(toolmanager, tools=default_tools):
     toolmanager : `.backend_managers.ToolManager`
         Manager to which the tools are added.
     tools : {str: class_like}, optional
-        The tools to add in a {name: tool} dict, see `add_tool` for more
-        info.
+        The tools to add in a {name: tool} dict, see
+        `.backend_managers.ToolManager.add_tool` for more info.
     """
 
     for name, tool in tools.items():
@@ -991,11 +1014,12 @@ def add_tools_to_container(container, tools=default_toolbar_tools):
     Parameters
     ----------
     container : Container
-        `backend_bases.ToolContainerBase` object that will get the tools added.
+        `.backend_bases.ToolContainerBase` object that will get the tools
+        added.
     tools : list, optional
         List in the form ``[[group1, [tool1, tool2 ...]], [group2, [...]]]``
         where the tools ``[tool1, tool2, ...]`` will display in group1.
-        See `add_tool` for details.
+        See `.backend_bases.ToolContainerBase.add_tool` for details.
     """
 
     for group, grouptools in tools:
